==> kap_lib-v1.28.3-mc1.20.1-sources.jar: net/kapitencraft/kap_lib/client/particle/animation/elements/GroupElement.java <==
package net.kapitencraft.kap_lib.client.particle.animation.elements;

import net.kapitencraft.kap_lib.client.particle.animation.core.ParticleConfig;
import net.kapitencraft.kap_lib.helpers.NetworkHelper;
import net.kapitencraft.kap_lib.registry.custom.particle_animation.ElementTypes;
import net.minecraft.network.FriendlyByteBuf;
import org.jetbrains.annotations.NotNull;

import java.util.Arrays;

public class GroupElement implements AnimationElement {
    private final AnimationElement[] elements;

    public GroupElement(AnimationElement[] elements) {
        this.elements = elements;
    }


    @Override
    public @NotNull Type getType() {
        return ElementTypes.GROUP.get();
    }

    @Override
    public int createLength(ParticleConfig config) {
        int max = 0;
        for (AnimationElement element : elements) {
            int check = element.createLength(config);
            if (check > max) max = check;
        }
        return max;
    }

    @Override
    public void tick(ParticleConfig object, int tick, double percentage) {
        for (AnimationElement element : elements) {
            element.tick(object, tick, percentage);
        }
    }

    public static class Type implements AnimationElement.Type<GroupElement> {

        @Override
        public GroupElement fromNW(FriendlyByteBuf buf) {
            AnimationElement[] elements = NetworkHelper.readArray(buf, AnimationElement[]::new, AnimationElement::fromNw);
            return new GroupElement(elements);
        }

        @Override
        public void toNW(FriendlyByteBuf buf, GroupElement value) {
            NetworkHelper.writeArray(buf, value.elements, AnimationElement::toNw);
        }
    }

    public static class Builder implements AnimationElement.Builder {
        private final AnimationElement.Builder[] builders;

        public Builder(AnimationElement.Builder... builders) {
            this.builders = builders;
        }

        @Override
        public AnimationElement build() {
            return new GroupElement(Arrays.stream(builders).map(AnimationElement.Builder::build).toArray(AnimationElement[]::new));
        }
    }
}

==> kap_lib-v1.28.5-mc1.21.1-sources.jar: net/kapitencraft/kap_lib/client/particle/animation/elements/GroupElement.java <==
package net.kapitencraft.kap_lib.client.particle.animation.elements;

import net.kapitencraft.kap_lib.client.particle.animation.core.ParticleConfig;
import net.kapitencraft.kap_lib.helpers.ExtraStreamCodecs;
import net.kapitencraft.kap_lib.registry.custom.particle_animation.ElementTypes;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.network.RegistryFriendlyByteBuf;
import net.minecraft.network.codec.ByteBufCodecs;
import net.minecraft.network.codec.StreamCodec;
import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Stream;

public class GroupElement implements AnimationElement {
    private final List<AnimationElement> elements;

    public GroupElement(List<AnimationElement> elements) {
        this.elements = elements;
    }


    @Override
    public @NotNull Type getType() {
        return ElementTypes.GROUP.get();
    }

    @Override
    public int createLength(ParticleConfig config) {
        int max = 0;
        for (AnimationElement element : elements) {
            int check = element.createLength(config);
            if (check > max) max = check;
        }
        return max;
    }

    @Override
    public void tick(ParticleConfig object, int tick, double percentage) {
        for (AnimationElement element : elements) {
            element.tick(object, tick, percentage);
        }
    }

    public static class Type implements AnimationElement.Type<GroupElement> {
        private static final StreamCodec<? super RegistryFriendlyByteBuf, GroupElement> STREAM_CODEC = AnimationElement.CODEC.apply(ByteBufCodecs.list()).map(GroupElement::new, e -> e.elements);

        @Override
        public StreamCodec<? super RegistryFriendlyByteBuf, GroupElement> codec() {
            return STREAM_CODEC;
        }
    }

    public static class Builder implements AnimationElement.Builder {
        private final AnimationElement.Builder[] builders;

        public Builder(AnimationElement.Builder... builders) {
            this.builders = builders;
        }

        @Override
        public AnimationElement build() {
            return new GroupElement(Arrays.stream(builders).map(AnimationElement.Builder::build).toList());
        }
    }
}
